#*****************************
# Python-frog
#   by Maarten van Gompel
#   Centre for Language Studies
#   Radboud University Nijmegen
#
#   Licensed under GPLv3
#****************************/

from libcpp.string cimport string
from libcpp.vector cimport vector
from libcpp cimport bool
from libc.stdint cimport *

from libfolia_classes cimport Document

cdef extern from "<iostream>" namespace "std":
    cdef cppclass ostream:
        ostream()

    cdef cppclass istream:
        pass

    extern ostream cout
    extern ostream cerr


cdef extern from "ticcutils/Configuration.h" namespace "TiCC":
    cdef cppclass Configuration:
        Configuration()
        bool fill( string filename )
        bool hasSection( string section )
        str configDir()
        string setatt(string&, string&, string&)
        string setatt(string&, string&)

cdef extern from "ticcutils/LogStream.h" namespace "TiCC":
    cdef cppclass LogStream:
        LogStream()
        LogStream(string prefix)

cdef extern from "ticcutils/CommandLine.h" namespace "TiCC":
    cdef cppclass CL_Options:
        CL_Options()
        void insert(string&, string&)
        void insert(char, string&, bool)
        bool is_present(string&)
        bool is_present(char)

cdef extern from "frog/Frog-util.h":
    cdef cppclass TimerBlock:
        pass

cdef extern from "frog/ucto_tokenizer_mod.h":
    cdef cppclass UctoTokenizer:
        Document * tokenizehelper( string)

cdef extern from "frog/FrogAPI.h":

    cdef cppclass FrogAPI:
        UctoTokenizer * tokenizer

        FrogAPI(CL_Options& options, LogStream * logstream, LogStream * debuglogstream)

        string Frogtostring(string s)

        string defaultConfigDir(string lang)
        string defaultConfigFile(string lang)



#embedsignature=True
#*****************************
# Python-ucto
#   by Maarten van Gompel
#   Centre for Language Studies
#   Radboud University Nijmegen
#
#   Licensed under GPLv3
#****************************/

from libcpp.string cimport string
from libcpp cimport bool
from libcpp.vector cimport vector
from cython.operator cimport dereference as deref, preincrement as inc
from cython import address
from libc.stdint cimport *
from libcpp.utility cimport pair
import os.path
import sys
cimport libfolia_classes
cimport frog_classes

try:
    from folia.main import Document as FoliaPyDocument
    HASFOLIAPY = True
except:
    try:
        from pynlpl.formats.folia import Document as FoliaPyDocument
        HASFOLIAPY = True
    except ImportError:
        HASFOLIAPY = False
        class FoLiAPyDocument: #dummy
            pass



cdef class Document:
    cdef frog_classes.Document capi



cdef class FrogOptions:
    """Options for Frog, passed as keyword arguments to the constructor. Also accessible like dictionary keys.

    You can pass as keyword arguments to the constructor any options similar to the command-line paramters accepted by Frog.
    Please see frog --help for a complete list.

        * ``skip`` ``[values]`` - Skip Tokenizer (t), Lemmatizer (l), Morphological Analyzer (a), Chunker (c), Multi-Word Units (m), Named Entity Recognition (n), or Parser (p)
        * ``id`` ``[string]`` - Document ID for FoLiA output

    Boolean parameters are passed like ``n=True``

    Old style-options: These are deprecated but supported for backward compatibility reasons:

        * ``tok`` - True/False - Do tokenisation? (default: True)
        * ``lemma`` - True/False - Do lemmatisation? (default: True)
        * ``morph`` - True/False - Do morpholigical analysis? (default: True)
        * ``deepmorph`` - True/False - Do morphological analysis in new experimental style? (default: False)
        * ``mwu`` - True/False - Do Multi Word Unit detection? (default: True)
        * ``chunking`` - True/False - Do Chunking/Shallow parsing? (default: True)
        * ``ner`` - True/False - Do Named Entity Recognition? (default: True)
        * ``parser`` - True/False - Do Dependency Parsing? (default: False)
        * ``xmlin`` - True/False - Input is FoLiA XML (default: False)
        * ``xmlout`` - True/False - Output is FoLiA XML (default: False)
        * ``docid`` - str - Document ID (for FoLiA)
        * ``numThreads`` - int - Number of threads to use (default: unset, unlimited)

    """
    cdef frog_classes.CL_Options capi
    cdef dict shadow
    cdef list skip

    def __init__(self, **kwargs):
        self.shadow = {} #shadow all settings in a dictionary
        self.skip = ["p"] #skip parser by default
        for key, value in kwargs.items():
            self[key] = value

    def __getitem__(self, key):
        key = key.replace("_","-").lower()
        if key in self.shadow:
            return self.shadow[key]
        else:
            raise KeyError("No such key: " + str(key))

    def get(self, key, default=False):
        key = key.replace("_","-").lower()
        if key in self.shadow:
            return self.shadow[key]
        else:
            return default

    def __setitem__(self, key, value):
        key = key.replace("_","-")
        self.shadow[key.lower()] = value
        if key.lower() in ('dotok','tok'):
            if not value: self.skip.append("t")
        elif key.lower() in ('dolemma','lemma'):
            if not value: self.skip.append("l")
        elif key.lower() in ('domorph','morph'):
            if not value: self.skip.append("a")
        elif key.lower() in ('dodaringmorph','daringmorph','deepmorph','dodeepmorph'):
            self.capi.insert(<string>"deepmorph",<string>"")
        elif key.lower() in ('domwu','mwu'):
            if not value: self.skip.append("m")
        elif key.lower() in ('doiob','iob','dochunking','chunking','shallowparsing'):
            if not value: self.skip.append("c")
        elif key.lower() in ('doner','ner'):
            if not value: self.skip.append("n")
        elif key.lower() in ('doparse','doparser','parse','parser'):
            if not value:
                self.skip.append("p")
            elif value:
                self.skip.remove("p")
        elif key.lower() in ('doxmlin','xmlin','foliain'):
            self.capi.insert(<char>"x", <string>"", False)
        elif key.lower() in ('doxmlout','xmlout','foliaout'):
            self.capi.insert(<char>"X", <string>"", False)
        elif key.lower() in ('debug','debugflag'):
            if value: self.capi.insert(<char>"d", <string>"1", False)
        elif key.lower() in ('docid','id'):
            self.capi.insert(<string>"id", <string>value)
        elif key.lower() in ('numthreads','threads'):
            self.capi.insert(<string>"threads",<string>value)
        else:
            if key == 'x':
                self.shadow['xmlin'] = True
            elif key == 'X':
                self.shadow['xmlout'] = True
            self.capi.insert(<string>key, <string>value)

    def finish(self):
        v = "".join(self.skip)
        self.capi.insert(<string>"skip", <string>v.encode('utf-8'))



cdef class Frog:
    cdef frog_classes.FrogAPI * capi
    cdef FrogOptions options
    cdef frog_classes.Configuration configuration
    cdef frog_classes.LogStream logstream
    cdef frog_classes.LogStream debuglogstream

    def __init__(self, FrogOptions options, configurationfile = "", overrides = None):
        """Initialises Frog, pass a FrogOptions instance and a configuration file, and optionally a dictionary containing overrides for the configuration."""

        options.finish()
        if overrides:
            assert(overrides, dict)
            for key, value in overrides.item():
                v = key + "=" + value
                options.capi.insert(<string>"override", <string>v)
        self.options = options

        if configurationfile:
            self.configuration.fill(configurationfile.encode('utf-8'))
        else:
            self.configuration.fill(self.capi.defaultConfigFile("nld"))

        self.capi = new frog_classes.FrogAPI(options.capi, &self.logstream, &self.debuglogstream)


    def process_raw(self, text):
        """Invokes Frog on the specified text, the text is considered one document. The raw results from Frog are returned as a string"""
        #cdef libfolia_classes.Document * doc = self.capi.tokenizer.tokenizehelper( text.encode('utf-8') )
        cdef string result = self.capi.Frogtostring(self._encode_text(text))
        r = result.decode('utf-8') #if (sys.version < '3' and type(text) == unicode) or (sys.version > '3' and type(text) == str) else result
        return r

    def parsecolumns(self, response):
        """Parse the raw Frog response"""
        if self.options.get('doDeepMorph'):
            columns = ('index','text','lemma','morph','compound','pos','posprob','ner','chunker','depindex','dep')
        else:
            columns = ('index','text','lemma','morph','pos','posprob','ner','chunker','depindex','dep')
        data = []
        for line in response.split('\n'):
            if not line.strip():
                if data:
                    data[-1]['eos'] = True
            else:
                item = {}
                for i, field in enumerate(line.split('\t')):
                    if field:
                        if columns[i] == 'posprob':
                            item[columns[i]] = float(field)
                        else:
                            item[columns[i]] = field
                data.append(item)
        return data


    def process(self, text):
        """Invokes Frog on the specified text. The text may be a string, or a folia.Document instance if Frog was instantiated with xmlin=True. If xmlout=False (default), the results from Frog are parsed into a list of dictionaries, one per token; if True, a FoLiA Document instance is returned"""
        if self.options.get('xmlin') and HASFOLIAPY and isinstance(text, FoliaPyDocument):
            text = str(text)
        elif not isinstance(text,str) and not (sys.version < '3' and isinstance(text,unicode)):
            raise ValueError("Text should be a string or FoLiA Document instance")

        if self.options.get('xmlout') and HASFOLIAPY:
            if HASFOLIAPY:
                data = self.process_raw(text)
                if not data:
                    raise ValueError("No data returned")
                elif data[0] != '<':
                    raise ValueError("Returned data is not XML, got: " + str(data[:25]))
                return FoliaPyDocument(string=data)
            else:
                raise Exception("Unable to return a FoLiA Document. FoLiAPy was not installed. Use process_raw() instead if you just want the XML output as string")
        else:
            return self.parsecolumns(self.process_raw(text))

    def _encode_text(self, text):
        if type(text) == str:
            return text.encode('utf-8')
        return text #already was bytes or python2 str




#!/usr/bin/env python
from distutils.core import setup, Extension
from Cython.Distutils import build_ext
import platform
import glob
import os


includedirs = []
libdirs = []
if platform.system() == "Darwin":
    #we are running on Mac OS X with homebrew, stuff is in specific locations:
    libdirs.append("/usr/local/opt/icu4c/lib")
    includedirs.append("/usr/local/opt/icu4c/include")
    libdirs.append("/usr/local/opt/libxml2/lib")
    includedirs.append("/usr/local/opt/libxml2/include")
    includedirs.append("/usr/local/opt/libxml2/include/libxml2")

#add some common default paths
includedirs += ['/usr/include/', '/usr/include/libxml2','/usr/local/include/' ]
libdirs += ['/usr/lib','/usr/local/lib']
if 'VIRTUAL_ENV' in os.environ:
    includedirs.insert(0,os.environ['VIRTUAL_ENV'] + '/include')
    libdirs.insert(0,os.environ['VIRTUAL_ENV'] + '/lib')
if 'INCLUDE_DIRS' in os.environ:
    includedirs = list(os.environ['INCLUDE_DIRS'].split(':')) + includedirs
if 'LIBRARY_DIRS' in os.environ:
    libdirs = list(os.environ['LIBRARY_DIRS'].split(':')) + libdirs

if platform.system() == "Darwin":
    extra_options = ["--stdlib=libc++",'-D U_USING_ICU_NAMESPACE=1']
else:
    extra_options = ['-D U_USING_ICU_NAMESPACE=1']

extensions = [ Extension("frog",
                [ "libfolia_classes.pxd", "frog_classes.pxd", "frog_wrapper.pyx"],
                language='c++',
                include_dirs=includedirs,
                library_dirs=libdirs,
                libraries=['frog','ucto','folia'],
                extra_compile_args=['--std=c++0x'] + extra_options,
                pyrex_gdb=True
                ) ]

setup(
    name = 'python-frog',
    version = '0.5.0',
    author = "Maarten van Gompel",
    author_email = "proycon@anaproy.nl",
    description = ("Python binding to FROG, an NLP suite for Dutch doing part-of-speech tagging, lemmatisation, morphological analysis, named-entity recognition, shallow parsing, and dependency parsing."),
    license = "GPL",
    keywords = "nlp computational_linguistics dutch pos lemmatizer",
    url = "https://github.com/proycon/python-frog",
    ext_modules = extensions,
    cmdclass = {'build_ext': build_ext},
    requires = ['frog (>=0.16)'],
    install_requires=['Cython'],
    classifiers=[
        "Development Status :: 4 - Beta",
        "Topic :: Text Processing :: Linguistic",
        "Programming Language :: Cython",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Operating System :: POSIX",
        "Intended Audience :: Developers",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
    ],
)



from libcpp.string cimport string
from libcpp.vector cimport vector
from libcpp cimport bool
from libc.stdint cimport *

cdef extern from "libfolia/folia.h" namespace "folia":
    cdef int BASE
    cdef int TextContent_t
    cdef int Text_t
    cdef int Word_t
    cdef int Str_t
    cdef int WordReference_t
    cdef int Event_t
    cdef int TimeSegment_t
    cdef int TimingLayer_t
    cdef int LineBreak_t
    cdef int WhiteSpace_t
    cdef int Sentence_t
    cdef int Paragraph_t
    cdef int Division_t
    cdef int Head_t
    cdef int Caption_t
    cdef int Label_t
    cdef int List_t
    cdef int ListItem_t
    cdef int Figure_t
    cdef int Quote_t
    cdef int Pos_t
    cdef int Lemma_t
    cdef int Phon_t
    cdef int Domain_t
    cdef int Sense_t
    cdef int Subjectivity_t
    cdef int Metric_t
    cdef int Correction_t
    cdef int AnnotationLayer_t
    cdef int SyntacticUnit_t
    cdef int Chunk_t
    cdef int Chunking_t
    cdef int Entity_t
    cdef int Entities_t
    cdef int Coreferences_t
    cdef int CoreferenceLink_t
    cdef int CoreferenceChain_t
    cdef int SyntaxLayer_t
    cdef int Semroles_t
    cdef int Semrole_t
    cdef int Morphology_t
    cdef int Morpheme_t
    cdef int ErrorDetection_t
    cdef int New_t
    cdef int Original_t
    cdef int Current_t
    cdef int Alternative_t
    cdef int Alternatives_t
    cdef int Description_t
    cdef int Gap_t
    cdef int Suggestion_t
    cdef int Content_t
    cdef int Feature_t
    cdef int SynsetFeature_t
    cdef int ActorFeature_t
    cdef int HeadFeature_t
    cdef int ValueFeature_t
    cdef int TimeFeature_t
    cdef int ModalityFeature_t
    cdef int LevelFeature_t
    cdef int BeginDateTimeFeature_t
    cdef int EndDateTimeFeature_t
    cdef int FunctionFeature_t
    cdef int PlaceHolder_t
    cdef int Dependencies_t
    cdef int Dependency_t
    cdef int Headwords_t
    cdef int DependencyDependent_t
    cdef int Alignment_t
    cdef int AlignReference_t
    cdef int Table_t
    cdef int TableHead_t
    cdef int Row_t
    cdef int Cell_t
    cdef int Lang_t
    cdef int XmlComment_t
    cdef int TokenAnnotation_t
    cdef int Structure_t
    cdef int AbstractTextMarkup_t
    cdef int TextMarkupString_t
    cdef int TextMarkupGap_t
    cdef int TextMarkupCorrection_t
    cdef int TextMarkupError_t
    cdef int TextMarkupStyle_t
    cdef int XmlText_t
    cdef int External_t
    cdef int Note_t
    cdef int Reference_t
    cdef int Part_t


    cdef cppclass FoliaElement:
        #FoliaElement * index(int) +KeyError
        int size()
        bool isinstance(int)
        string xmltag()


    cdef cppclass Document:
        Document()
        Document(string kwargs)

        bool readFromFile(string& filename) nogil
        bool readFromString(string& xml) nogil

        bool save(string&) nogil

        string toXml() nogil
